function [vModelResults,vClassifyResults,vT,vM,vS] = crossvalidation_clustering(rule, SDL, v, opt)
%CROSSVAL K-fold cross validation.
%   [C, POST] = CROSSVAL(RULE, X, K) returns the leave-one-out
%   cross-validation classification estimates of the observations
%   described by the feature matrix X and the class index vector K
%   given the name of the classifier RULE. RULE must be a string
%   containing the name of a CLASSIFIER object such as one of the
%   descendents of the class 'classifier'. X is an n by p matrix where
%   n is the number of observations and p is the number of
%   variables. K is a length n vector of class indeces in which each
%   element gives the group to which the corresponding row in X
%   belongs. C contains the estimated classificiations in a length n
%   class index vector and POST is a n by g matrix of posterior
%   probabilities.
%
%   [C, POST, V] = CROSSVAL(RULE, X, K, NSETS) where NSETS is a scalar
%   value no more than n perfoms NSETS-fold cross-validation with
%   randomly chosen sets of approximately equal size. The return
%   argument V is a vector of indeces indicating the cross-validation
%   group to which each row of X and K were assigned. If NSETS is a
%   vector of indeces, it is used as the cross-validition sets
%   instead. Empty sets are not permitted in NSETS.
%
%   CROSSVAL(RULE, X, K, OPTS) allows optional arguments to be passed
%   in the fields of OPTS. The only field that is used by CROSSVAL is
%   NSETS. Additionally, OPTS is passed to the CLASSIFIER constructor
%   as well as the function CLASSIFY for the appropriate CLASSIFIER
%   class so that options may be assigned to those funcitons as well.

%   Copyright (C) Michael Kiefte 1998.

%   $Log$
%   call: 
%   [vModelResults,vClassifyResults] =
%   crossvalidation_clustering(@kngas_ny_cv, oSDL, 5, {3,10,0.1,3/2,0.05,'nystroem',0.1,'sparse_beta',10,'sparse_sigma',0.001,'few_memory',0} )
%
X = SDL.data;
k = SDL.labels;

error(nargchk(3, 4, nargin))
[n p] = size(X);
if nargin < 3
  v = [];
end

if nargin < 4
  opt = [];
end

if isempty(v)
  V = 1:n;
  v = n;
elseif length(v) == 1
  if ~isa(v, 'double') | ~isreal(v) | v <= 0 | round(v) ~= v
    error('V must be a positive, non-zero integer.')
  elseif v > n
    error('V must be less than the number of observations.')
  elseif v == n
    V = 1:n;
  else
    w = v*fix(n/v);
    r = randperm(v);
    V(randperm(n)) = [reshape(repmat(1:v, w/v, 1), w, 1); ...
		      r(1:n-w)'];
  end
else
  if ~isa(v, 'double') | ~isreal(v) | prod(size(v)) ~= length(v) | ...
	any(round(v) ~= v | v <= 0 | isinf(v))
    error(['Cross-validation set indeces V must be a vector of' ...
	   ' positive, finite, non-zero integers.'])
  elseif length(v) ~= n
    error(['Length of set indeces V must be same as number of' ...
	   ' observations in X and K.'])
  elseif ~all(sum(sparse(1:n, v, 1)))
    error('Set indeces V may not have empty groups.')
  end
  
  V = v;
  v = max(V);
end
  
c = zeros(n, 1);
vModelResults    = [];
vClassifyResults = [];
vT = []; vM = [];
for i = 1:v
  s = warning;
  warning off
  SDLCurrent = SDL;
  SDLCurrent.data   = SDLCurrent.data(V ~= i, :);
  SDLCurrent.labels = SDLCurrent.labels(V ~= i, :);
  tic;
  oModel = feval(rule, SDLCurrent, opt);
  vT(i) = toc;
  vModelResults = [vModelResults oModel];
  warning(s);
  SDLTest = SDL;
  SDLTest.data   = SDLTest.data(V == i, :);
  SDLTest.labels = SDLTest.labels(V == i, :);  
  [vLabelsCluster,dAcc,mClasses] = PostLabel(oModel,SDLTest);
  vClassifyResults = [vClassifyResults dAcc];
end
vM=zeros(length(vModelResults),1);
O = ones(size(vModelResults(1).G)); S=whos('O'); iRef=S.bytes;for(i=1:length(vModelResults)), temp=vModelResults(i).G; S=whos('temp'); vM(i) = S.bytes./iRef; end
vS=arrayfun(@(x) (nnz(x.G)/numel(x.G)),vModelResults,'UniformOutput',true);








